' BLITZMAX STD-LIB
' ~~  GEOMETRY  ~~

Rem
bbdoc: 2D point class.
End Rem
Type StdPoint Extends StdVector2d
	Rem
	bbdoc: Returns a new StdPoint instance.
	EndRem
	Function Create:StdPoint( x_:Double = .0, y_:Double = .0, z_:Double = .0 )
		Local ret_:StdPoint = New StdPoint
		ret_.x = x_
		ret_.y = y_
		
		Return ret_
	EndFunction
	
	Rem
	bbdoc: Returns a new StdPoint from an array [2]
	EndRem
	Function FromArray:StdPoint( floats_:Float[] )
		If floats_.length < 2 Throw "StdPoint.FromArray: Array size < 2"
		Return StdPoint.Create(floats_[0], floats_[1])
	EndFunction
	
	Rem
	bbdoc: offset the recievers coordinates by x and y.
	EndRem
	Method offset:StdPoint( amX:Float, amY:Float )
		Self.x :+ amX
		Self.y :+ amY
	EndMethod
	
	Rem
	bbdoc: reflects the reciever about the x-axis.
	EndRem
	Method reflectAboutXAxis()
		Self.x = -Self.x
	EndMethod
	
	Rem
	bbdoc: reflects the reciever about the y-axis.
	EndRem
	Method reflectAboutYAxis()
		Self.y = -Self.y
	EndMethod
	
	Rem
	bbdoc: reflects the reciever about its origin.
	EndRem
	Method reflectAboutOrigin()
		Self.x = -Self.x
		Self.y = -Self.y
	EndMethod
	
	Rem
	bbdoc: returns YES if the recievers coordinates are equalt to the give points coordinates.
	EndRem
	Method isEqualToPoint:Int( p:StdPoint )
		If Self.x = p.x And Self.y = p.y
			Return True
		Else
			Return False
		EndIf
	EndMethod
		
	Method description:String()
		Return "{" + x + ", " + y + "}"
	EndMethod
	
	Method copy:StdPoint()
		Return StdMakePoint( x, y )
	EndMethod
EndType

Function StdMakePoint:StdPoint( x:Float, y:Float )
	Return StdPoint.Create( x, y )
EndFunction

Rem
bbdoc: StdSize
about: 2D size class.
End Rem
Type StdSize Extends StdObject

	Field width:Float
	Field height:Float
	
	Rem
	bbdoc: returns a new <i>StdSize</i> instance.
	EndRem
	Function Create:StdSize( width_:Float, height_:Float )
		Local ret_:StdSize = New StdSize
		ret_.Width = width_
		ret_.Height = height_
		
		Return ret_
	EndFunction
	
	Rem
	bbdoc: Returns a new StdSize from an array [2]
	EndRem
	Function FromArray:StdSize( floats_:Float[] )
		If floats_.length < 2 Throw "StdSize.FromArray: Array size < 2"
		Return StdSize.Create(floats_[0], floats_[1])
	EndFunction
	
	Rem
	bbdoc: Returns a new empty StdSize instance.
	EndRem
	Function size:StdSize()
		Return New StdSize
	EndFunction
	
	Rem
	bbdoc: Set the values from the given size
	EndRem
	Method set( size_:StdSize )
		width = size_.width
		height = size_.height
	EndMethod
	
	Method isEqualToSize:Int( s:StdSize )
		If Self.width = s.width And Self.height = s.height
			Return True
		Else
			Return False
		EndIf
	EndMethod
	
	Method description:String()
		Return "{" + width + ", " + height + "}"
	EndMethod
	
	Method copy:StdSize()
		Return StdSize.Create( width, height )
	EndMethod
EndType

Function StdMakeSize:StdSize( width:Float, height:Float )
	Return StdSize.Create( width, height )
EndFunction

Rem
bbdoc: StdRect
about: 2D rect class.
End Rem
Type StdRect Extends StdObject
	
	Field _p:StdVector2d[4] 'private, only for rotation

	Field origin:StdPoint
	Field size:StdSize
	
	Method New()
		For Local i:Int = 0 Until 4
			_p[i] = New StdPoint
		Next
	EndMethod
	
	Rem
	bbdoc: returns a new <i>StdRect</i> instance.
	EndRem
	Function Create:StdRect( x_:Float, y_:Float, width_:Float, height_:Float )
		Local ret_:StdRect = New StdRect
		ret_.origin = StdPoint.Create(x_, y_)
		ret_.size = StdSize.Create(width_, height_)
		
		Return ret_
	EndFunction
	
	Rem
	bbdoc: Create a new StdRect, initialized to position {0,0} and the dimensions of the given @size.
	End Rem
	Function FromSize:StdRect(size_:StdSize)
		Return StdRect.Create(0, 0, size_.width, size_.height)
	End Function
	
	Rem
	bbdoc: Returns a new StdRect from an array [4]
	EndRem
	Function FromArray:StdRect( floats_:Float[] )
		If floats_.length < 4 Throw "StdRect.FromArray: Array size < 4"
		Return StdRect.Create(floats_[0], floats_[1], floats_[2], floats_[3])
	EndFunction
	
	Rem
	returns the recievers width.
	EndRem
	Method width:Float()
		If Self.size.width < 0.0 
			Return 0.0
		Else
			Return Self.size.width
		EndIf
	EndMethod
	
	Rem
	returns the recievers height.
	EndRem
	Method height:Float()
		If Self.size.height < 0.0 
			Return 0.0
		Else
			Return Self.size.height
		EndIf
	EndMethod
	
	Rem
	returns the recievers x coordinate.
	EndRem
	Method minX:Float()
		Return Self.origin.x
	EndMethod
	
	Rem
	returns the recievers y coordinate.
	EndRem
	Method minY:Float()
		Return Self.origin.y
	EndMethod
	
	Method maxX:Float()
		Return Self.origin.x + Self.size.width
	EndMethod
	
	Method maxY:Float()
		Return Self.origin.y + Self.size.height
	EndMethod
	
	Method midX:Float()
		Return Self.origin.x + Self.size.width/2.0
	EndMethod
	
	Method midY:Float()
		Return Self.origin.y + Self.size.height/2.0
	EndMethod
	
	Rem
	returns the recievers x coordinate.
	EndRem
	Method x:Float()
		Return minX()
	EndMethod
	
	Rem
	returns the recievers y coordinate.
	EndRem
	Method y:Float()
		Return miny()
	EndMethod
	
	Rem
	bbdoc: Set the values from the given size
	EndRem
	Method set( rect_:StdRect )
		origin.set(rect_.origin)
		size.set(rect_.size)
	EndMethod
	
	'-------
	'polygon
	'-------
	
	Rem
	returns a float array [8] for use with @{drawPoly}.
	EndRem
	Method polyArray:Float[]()
		Local f:Float[] = [minX(), minY(), minX(), maxY(), maxX(), maxY(), maxX(), minY()]
		Return f
	EndMethod
	
	Rem
	returns a float array [16] for use with @{StdDrawLineStrip}.
	EndRem
	Method linestripArray:Float[]()
		Local f:Float[] = [minX(), minY(), minX(), maxY(), minX(), maxY(), maxX(), maxY(), maxX(), maxY(), maxX(), minY(), maxX(), minY(), minX(), minY()]
		Return f
	EndMethod
	
	'---------------
	'awesome methods
	'---------------
	
	Method divide:StdRect[,]( timesx_:Int, timesy_:Int )
		If timesx_ <= 0 Then timesx_ = 1
		If timesy_ <= 0 Then timesy_ = 1
		Local arr_:StdRect[ timesx_, timesy_ ]
		Local width_:Float = width() / Float(timesx_)
		Local height_:Float = height() / Float(timesy_)
		
		For Local x_:Int = 0 Until timesx_
			For Local y_:Int = 0 Until timesy_
				arr_[x_, y_] = StdRect.Create( Float(Self.x() + Float(x_) * width_), Float(Self.y() + Float(y_) * height_), width_, height_ )
			Next
		Next
		
		Return arr_
	EndMethod
	
	Method intersectWithRect:StdRect( rect_:StdRect )
		If Not collidesWithRect( rect_ ) Then Return StdRect.Create( 0.0, 0.0, 0.0, 0.0 )
	
		Local x_:Float, y_:Float, w_:Float,h_:Float
		
		'x
		If x() < rect_.x()
			x_ = max2f(x(), rect_.x())
			w_ = width() - ( max2f(rect_.x(), x()) - min2f(rect_.x(), x()) )
			
		Else
			x_ = x()
			w_ = rect_.MaxX() - x()
			
		End If
		If w_ > rect_.Width() And w_> Width()
			w_ = .0
		
		ElseIf w_ > rect_.Width()
			w_ = rect_.Width()
			
		ElseIf w_ > Width()
			w_ = Width()
			
		End If
		
		'y
		If y() < rect_.y()
			y_ = max2f(y(), rect_.y())
			h_ = height() - ( max2f(rect_.y(), y()) - min2f(rect_.y(), y()) )
			
		Else
			y_ = y()
			h_ = rect_.MaxY() - y()
			
		End If
		If h_ > rect_.Height() And h_> Height()
			h_ = .0
		
		ElseIf h_ > rect_.Height()
			h_ = rect_.Height()
			
		ElseIf h_ > Height()
			h_ = Height()
			
		End If
	
		Return StdRect.Create(x_, y_, w_, h_)
	End Method
	
	'---------
	'collision
	'---------
	
	Rem
	bbdoc: returns if the rect collides with the given point.
	EndRem
	Method collidesWithPoint:Int( point_:StdPoint )
		If point_.x >= Self.MinX() And point_.y >= Self.MinY() And ..
		point_.x <= Self.MaxX() And point_.y <= Self.MaxY() Then 
			Return True
		Else
			Return False
		EndIf
	EndMethod
	
	Rem
	bbdoc: Collides with Point, rotated
	EndRem
	Method collidesWithPointRotated:Int( rot_:Float, handlex_:Float, handley_:Float, point_:StdPoint )
		_p[0].set(origin)
		_p[1].set(origin) ; _p[1].y = maxY()
		_p[2].set(origin) ; _p[2].x = maxX() ; _p[2].y = maxY()
		_p[3].set(origin) ; _p[3].x = maxX()
		
		For Local i:Int = 0 Until 4
			_p[i].rotate(rot_, origin.x+handlex_, origin.y+handley_)
		Next
		
		If point_.inTriangle(_p[0], _p[1], _p[2]) Or point_.inTriangle(_p[0], _p[3], _p[2]) Return True
		Return False
	EndMethod
	
	Rem
	bbdoc: returns if the rect collides with the given rect.
	EndRem
	Method collidesWithRect:Int( rect_:StdRect )
		If maxX() >= rect_.x() And maxY() >= rect_.y() And x() <= rect_.maxX() And y() <= rect_.maxY() 
			Return True
		Else
			Return False
		EndIf
	EndMethod
	
	Rem
	bbdoc: returns if the rect is equal to the given rect.
	EndRem
	Method isEqualToRect:Int( r:StdRect )
		If Self.origin.isEqualToPoint( r.origin ) And Self.size.isEqualToSize( r.size )
			Return True
		Else
			Return False
		EndIf
	EndMethod
	
	Method isEqual:Int( o_:StdObject )
		Return isEqualToRect( StdRect( o_ ) )
	EndMethod
	
	Method description:String()
		Return "{" + origin.description() + ", " + size.description() + "}"
	EndMethod
	
	Method copy:StdRect()
		Return StdMakeRect( origin.x, origin.y, size.width, size.height )
	EndMethod

EndType

Function StdMakeRect:StdRect( x:Float, y:Float, width:Float, height:Float )
	Return StdRect.Create( x, y, width, height )
EndFunction