' BLITZMAX STD-LIB
' ~~   VECTOR   ~~

Rem
bbdoc: Two-dimensional Double Precision vector class.
about: Basic operations such as adding, multiplying and scaling are supported.
EndRem
Type StdVector2d Extends StdObject
	Field x:Double
	Field y:Double

	Function Create:StdVector2d( x_:Double = .0, y_:Double = .0, z_:Double = .0 )
		Local ret_:StdVector2d = New StdVector2d
		ret_.x = x_
		ret_.y = y_
		
		Return ret_
	EndFunction

	Method set( vec_:StdVector2d )
		x = vec_.x
		y = vec_.y
	EndMethod
	
	Method get( x_:Double Ptr, y_:Double Ptr, z_:Double Ptr = Null )
		x_[0] = x
		y_[0] = y
	EndMethod

	Method getX:Double()
		Return x
	EndMethod
	
	Method setX( x_:Double )
		x = x_
	EndMethod

	Method getY:Double()
		Return y
	EndMethod
	
	Method setY( y_:Double )
		y = y_
	EndMethod
	
	Method getZ:Double()
		Return 0
	EndMethod
	
	Method scale( factor_:Double )
		x:* factor_
		y:* factor_
	End Method
	
	Method multiply( factor_:StdVector2d )
		x:* factor_.x
		y:* factor_.y
	End Method
	
	Method assign( vec_:StdVector2d )
		x = vec_.x
		y = vec_.y
	End Method
	
	Method add( vec_:StdVector2d )
		x:+ vec_.x
		y:+ vec_.y
	End Method
	
	Rem
	bbdoc: subtracts (-) the given point.
	EndRem
	Method subtract( p:StdVector2d )
		Self.x :- p.x
		Self.y :- p.y
	EndMethod
	
	Rem
	bbdoc: Get the distance of this point another one
	End Rem
	Method distanceTo:Double( point_:StdVector2d )
		Return Sqr((point_.x - x)^2.0 + (point_.y - y)^2.0)
	End Method
	
	Method isEqual:Int( object_:StdObject )
		If StdVector2d(object_)
			Return ..
				x = StdVector2d(object_).x And ..
				y = StdVector2d(object_).y And ..
				getZ() = StdVector2d(object_).getZ()
				
		End If
	EndMethod
	
	Method rotate( angle_:Float, cx_:Float = .0, cy_:Float = .0 )
		Local sin_:Double = Sin(angle_)
		Local cos_:Double = Cos(angle_)
		
		x:- cx_
		y:- cy_
		Local x_:Float = ( x * cos_ - y * sin_ ) + cx_
		Local y_:Float = ( y * cos_ + x * sin_ ) + cy_
		
		x = x_
		y = y_
	End Method
	
	Method inTriangle:Int( a_:StdVector2d, b_:StdVector2d, c_:StdVector2d )
		Local v0_:StdVector3d = StdVector3d.Create(c_.x, c_.y, c_.getZ()) 
		Local v1_:StdVector3d = StdVector3d.Create(b_.x, b_.y, b_.getZ())
		Local v2_:StdVector3d = StdVector3d.Create(x, y, getZ())
		
		v0_.subtract(a_)
		v1_.subtract(a_)
		v2_.subtract(a_)
		
		Local dot00_:Double = v0_.dotP(v0_)
		Local dot01_:Double = v0_.dotP(v1_)
		Local dot02_:Double = v0_.dotP(v2_)
		Local dot11_:Double = v1_.dotP(v1_)
		Local dot12_:Double = v1_.dotP(v2_)
		
		Local divisor_:Double = 1.0 / ( dot00_ * dot11_ - dot01_ * dot01_ )
		Local u_:Double = ( dot11_ * dot02_ - dot01_ * dot12_ ) * divisor_
		Local v_:Double = ( dot00_ * dot12_ - dot01_ * dot02_ ) * divisor_
		
		Return (u_ >= .0) And (v_ >= .0) And (u_ + v_ <= 1.0)
	End Method

	Method Compare:Int(withObject:Object)
		If StdVector2d(withObject)
			Return Not StdVector2d(withObject).isEqual(Self)
		Else
			Return Super.compare(withObject)
		End If
	End Method
EndType

Rem
bbdoc: Three-dimensional Double Precision vector class.
about: Basic operations such as adding, multiplying and scaling are supported.
EndRem
Type StdVector3d Extends StdVector2d
	Field z:Double

	Function Create:StdVector3d( x_:Double = .0, y_:Double = .0, z_:Double = .0 )
		Local ret_:StdVector3d = New StdVector3d
		ret_.x = x_
		ret_.y = y_
		ret_.z = z_
		
		Return ret_
	EndFunction
	
	Method getZ:Double()
		Return z
	EndMethod
	
	Method setZ( z_:Double )
		z = z_
	EndMethod
	
	Method set( vec_:StdVector2d )
		Super.set(vec_)
		z = vec_.getZ()
	End Method
	
	Method scale( factor_:Double )
		Super.scale(factor_)
		z:* factor_
	End Method
	
	Method multiply( factor_:StdVector2d )
		Super.multiply(factor_)
		z:* factor_.getZ()
	End Method
	
	Method assign( vec_:StdVector2d )
		Super.assign(vec_)
		z = vec_.getZ()
	End Method
	
	Method add( vec_:StdVector2d )
		Super.add(vec_)
		z:+ vec_.getZ()
	End Method
	
	Method subtract( p_:StdVector2d )
		Super.subtract(p_)
		z:- p_.getZ()
	End Method
	
	Method dotP:Double(p_:StdVector2d)
		Return x*p_.x + y*p_.y + z*p_.getZ()
	End Method
	
	Method crossP(p_:StdVector2d)
		x = ( ( y* p_.getZ() )-( getZ()* p_.y ) )
		y = -( ( x* p_.getZ() )-( getZ()* p_.x ) )
		z = ( ( x* p_.y )-( y* p_.x ) )
	End Method
EndType